import unittest
from PyInvestor.market import *

class MarketTest(unittest.TestCase):


    def test_crypto(self):
        Crypto()

    
    def test_upcomingipo(self):
        UpcomingIPO()

    
    def test_todayipo(self):
        TodayIPO()

    
    def test_gainers(self):
        Gainers()

    
    def test_losers(self):
        Losers()


    def test_mostactive(self):
        MostActive()


    def test_infocus(self):
        InFocus()


    def test_newsmarket(self):
        NewsMarket(last=50)


    def test_marketohlc(self):
        MarketOHLC()


    def test_previous(self):
        Previous()


    def test_sectorperformance(self):
        SectorPerformance()

    


if __name__ == '__main__':
    unittest.main()



from PyInvestor.utils import IEX_URL, _endpointmarket, _correctdate
import requests
import pandas as pd
from pandas.io.json import json_normalize



                
def Crypto():
    """ returns an array of quotes for all cryptocurrencies supported by the IEX API. 
    """
    response = _endpointmarket('crypto')
    df = pd.DataFrame(response)
    _correctdate(df)
    return df


    
def UpcomingIPO():
    """ list of upcoming IPOs scheduled for the current
    and next month. The response has to
    """
    response = _endpointmarket('upcoming-ipos')
    rawData = response['rawData']
    viewData = response['viewData']
    df_raw = pd.DataFrame(rawData) # you have to take out some rows
    df_view =  pd.DataFrame(viewData)
    _correctdate(df_raw)
    _correctdate(df_view)
    return df_raw, df_view



def TodayIPO():
    """ list of today IPOs scheduled for the current
    and next month. The response has to
    """
    response = _endpointmarket('today-ipos')
    rawData = response['rawData']
    viewData = response['viewData']
    df_raw = pd.DataFrame(rawData) # you have to take out some rows
    df_view =  pd.DataFrame(viewData)
    _correctdate(df_raw)
    _correctdate(df_view)
    return df_raw, df_view



def Gainers():
    """ Biggest gainers in the market
    """
    response = _endpointmarket('list/gainers')
    df = pd.DataFrame(response)
    _correctdate(df)
    return df



def Losers():
    """ Biggest loosers in the market
    """
    response = _endpointmarket('list/losers')
    df = pd.DataFrame(response)
    _correctdate(df)
    return df



def MostActive():
    """ Most actively traded in the market
    """
    response = _endpointmarket('list/mostactive')
    df = pd.DataFrame(response)
    _correctdate(df)
    return df



def InFocus():
    """ In Focus market shares
    """
    response = _endpointmarket('list/infocus')
    df = pd.DataFrame(response)
    _correctdate(df)
    return df



def NewsMarket(last):
    """ Last news on the market. Last must belong between 1 to 50
    """
    if (last < 1) or (last > 50):
        raise ValueError('Wrong value - "last" must be between 1 - 50')
    else:
        response = _endpointmarket('news/last/%s' %last)
        df = pd.DataFrame(response)
        return df


    
def MarketOHLC():
    """ Returns the official open and close for a given symbol$
    
    - Not yet sure if it is the best to show the data 
    """
    response = _endpointmarket('ohlc')
    df = json_normalize(response)
    _correctdate(df)
    return df



def Previous():
    """ Returns previous day adjusted price data for the whole market
    """
    
    response = _endpointmarket('previous')
    data = []
    for i in response.keys():
        data.append(response[i])

    df = pd.DataFrame(data)
    _correctdate(df)
    return df



def SectorPerformance():
    """ Returns previous sector performance
    """

    response = _endpointmarket('sector-performance')
    df = json_normalize(response)
    _correctdate(df)
    return df.drop(['type'], axis=1)







    



"""
__author__ = Fabio Capela

"""
import requests
import pandas as pd



IEX_URL = "https://api.iextrading.com/1.0" # the universal link to get access to the endpoint
time_columns = ["openTime","closeTime", "latestUpdate", "iexLastUpdated", "delayedPriceTime",
                "extendedPriceTime","processedTime",  "lastUpdated", "time"]    # all the columns that have a timestamp
timerange_split = ['5y', '2y', '1y', 'ytd', '6m', '3m', '1m']  # time range for the splits 
timerange_chart = ['5y', '2y', '1y', 'ytd', '6m', '3m', '1m', '1d', 'date', 'dynamic']  # time range for the timeseries/charts



def _endpoint(key, symbol, endpoint, **kwargs):
    """ returns the json related to a particular endpoint. key corresponds to the differents location of the API endpoints.
    """
    request_url = "%s/%s/%s/%s"%(IEX_URL, key, symbol, endpoint)
    response =  requests.get(request_url, **kwargs)
    return response.json()


def _endpointmarket(symbol, **kwargs):
    """ returns the json related to a particular market endpoitn
    """
    request_url = "%s/stock/market/%s"%(IEX_URL,symbol)
    response = requests.get(request_url, **kwargs)
    return response.json()


def _correctdate(df):
    for col in time_columns:
        if col in df.columns:
            df[col] = pd.to_datetime(df[col], unit='ms')
        



from PyInvestor.utils import IEX_URL, timerange_split, timerange_chart , _endpoint, _correctdate
import requests
import pandas as pd
import collections

"""
TODOs

-  implement a proper way to deal with out of connection requests
"""



class Stock:
    """Gathers data from the IEX endpoints for only one stock
    """

    def __init__(self, symbol):
        """Initialization of the class Stock
        """
        self.symbol = symbol.upper()
        self.key = 'stock'
        

    def Company(self):
        """ returns information related to the company
        """
        response = _endpoint(self.key, self.symbol, 'company')
        df = pd.DataFrame(response)
        df = df.drop(['tags'], axis=1)
        return df.drop_duplicates()
    

    def DelayedQuote(self):
        """ returns the 15 minute delayed market quote 
        """
        response = _endpoint(self.key, self.symbol, 'delayed-quote')
        df = pd.Series(response).to_frame().T
        _correctdate(df)
        return df

    
    def Dividends(self, timerange):
        """ returns the historical dividends based on the historical market data
        """
        if timerange not in timerange_split:
            raise ValueError('%s not a valid value. Please select: "5y","2y","1y","ytd","6m","3m","1m"')
        else:
            response = _endpoint(self.key, self.symbol, 'dividends/%s' %timerange)
        return pd.DataFrame(response)
        

    def Earnings(self):
        """ returns data from the four most recent reported quarters
        """
        response = _endpoint(self.key, self.symbol, 'earnings')
        return pd.DataFrame(response['earnings'])

    
    def EffectiveSpread(self):
        """ returns an array of effective spread, eligible volume, and price 
            improvement of a stock, by market. Effective spread is designed to 
            measure marketable orders executable in relation to the market 
            center's quoted spread and takes into account hidden and midpoint
            liquidity available at each market center.
        """
        response =  _endpoint(self.key, self.symbol, 'effective-spread')
        return pd.DataFrame(response)

    
    def Financials(self):
        """ returns income statement, balance sheet, and cash flow data from the four
            most recent reported quarters.
        """
        response = _endpoint(self.key, self.symbol, 'financials')
        return pd.DataFrame(response['financials'])


    def Stats(self):
        """ returns certain important number in relation with a stock
        """
        response = _endpoint(self.key, self.symbol, 'stats')
        return pd.Series(response).to_frame().T

    
    def LargestTrades(self):
        """ returns 15 minute delayed, last sale eligible trades
        """ 
        response = _endpoint(self.key, self.symbol, 'largest-trades')
        df = pd.DataFrame(response)
        _correctdate(df)
        return df


    def News(self, lastndays=10):
        if (lastndays > 50) or (lastndays < 1):
            raise ValueError('Value of last is not correct. It must in between [1,50]')
        else:
            response = _endpoint(self.key, self.symbol, 'news/last/%s' %lastndays)
            df = pd.DataFrame(response)
        return df
    
        
    def OHLC(self): 
        """ returns the official open, high, low and close for a given symbol with open and/or close official listing time 
        """
        response = _endpoint(self.key, self.symbol, 'ohlc')
        dic = collections.defaultdict()
        dic[self.symbol] = {}
        dic[self.symbol]['open'] = response['open']['price']
        dic[self.symbol]['close'] = response['close']['price']
        dic[self.symbol]['high'] = response['high']
        dic[self.symbol]['low'] = response['low']
        dic[self.symbol]['close_time'] = response['close']['time']
        dic[self.symbol]['open_time'] = response['open']['time']
        df = pd.DataFrame(dic)
        _correctdate(df)
        return df

    
    def Previous(self):
        """ returns previous day adjusted price data for a single stock
        """
        response = _endpoint(self.key, self.symbol, 'previous')
        return pd.DataFrame(response, index=[response['symbol']])


    def Price(self):
        """ returns a single number, corresponding to the IEX real time price, the 15 minute delayed market price, 
        or the previous close price 
        """
        return _endpoint(self.key, self.symbol, 'price')


    def Quote(self, displayPercent=False):
        """ returns several quoting prices such as calculationPrice, latestPrice, delayedPrice
        Option: displayPercent -- all percentage values will be multiplied by a factor 100
        """
        if displayPercent == False:
            response = _endpoint(self.key, self.symbol, 'quote')
        else:
            response = _endpoint(self.key, self.symbol, 'quote?displayPercent=true')
        df = pd.Series(response).to_frame().T
        _correctdate(df)
        return df

    
    def Relevant(self):
        """ similar to peers endpoint, except this will return most active market symbols when peers
        are not available.
        """
        response = _endpoint(self.key, self.symbol, 'relevant')
        return response['symbols']


    def Splits(self, timerange):
        """ returns the different splits that occured for a particular range of dates "timerange"
        """
        if timerange not in timerange_split:
            raise ValueError('%s not  a valid value, please enter: "5y", "2y", "1y", "ytd", "6m", "3m", "1m"' %timerange)
        else:
            response = _endpoint(self.key, self.symbol, 'splits/%s' %timerange)
        return pd.DataFrame(response)


    def Tags(self):
        response = _endpoint(self.key, self.symbol, 'company')
        return response['tags']

    
    def TimeSeries(self, timerange='1m'):
        """ returns the historically adjusted market-wide data based on the timerange.
            this turns out to be the same as the chart endpoint of IEX API.
        """
        if timerange not in timerange_chart:
            raise ValueError('%s not a valid value, please enter: "5y", "2y", "1y", "ytd", "6m", "3m", "1m", "1d", "date", "dynamic"' %timerange)
        else:
            response = _endpoint(self.key, self.symbol, 'time-series/%s' %timerange) # still to check if kwargs work
        return pd.DataFrame(response)
            

    def VolumebyVenue(self):
        """ returns 15 minute delayed and 30 day average consolidated volume
            percentage of a stock, by market. This call will always return 13 values, and will be 
            sorted in ascending order by current day trading volume percentage
        """
        response =  _endpoint(self.key, self.symbol, 'volume-by-venue')
        return pd.DataFrame(response)
    
        
    




import unittest
from PyInvestor.stock import Stock


class StockTest(unittest.TestCase):
    
        
    def test_company(self):
        AMZN = Stock('AMZN')
        AMZN.Company()

    
    def test_delayedquote(self):
        AMZN = Stock('AMZN')
        AMZN.DelayedQuote()

    
    def test_dividends(self):
        AMZN = Stock('AMZN')
        AMZN.Dividends('1y')

    
    def test_earnings(self):
        AMZN = Stock('AMZN')
        AMZN.Earnings()


    def test_effectivespread(self):
        AMZN = Stock('AMZN')
        AMZN.EffectiveSpread()


    def test_financials(self):
        AMZN = Stock('AMZN')
        AMZN.Financials()


    def test_stats(self):
        AMZN = Stock('AMZN')
        AMZN.Stats()


    def test_largestrades(self):
        AMZN = Stock('AMZN')
        AMZN.LargestTrades()


    def test_news(self):
        AMZN = Stock('AMZN')
        AMZN.News(lastndays = 20)
        AMZN.News()

    def test_ohlc(self):
        AMZN = Stock('AMZN')
        AMZN.OHLC()


    def test_previous(self):
        AMZN = Stock('AMZN')
        AMZN.Previous()


    def test_price(self):
        AMZN = Stock('AMZN')
        AMZN.Price()


    def test_quote(self):
        AMZN = Stock('AMZN')
        AMZN.Quote(displayPercent=True)
        AMZN.Quote()


    def test_relevant(self):
        AMZN = Stock('AMZN')
        AMZN.Relevant()


        
    def test_splits(self):
        AMZN = Stock('AMZN')
        AMZN.Splits('5y')


    def test_tags(self):
        AMZN = Stock('AMZN')
        AMZN.Tags()


    def test_timeseries(self):
        AMZN = Stock('AMZN')
        AMZN.TimeSeries('5y')


    def test_volumebyvenue(self):
        AMZN = Stock('AMZN')
        AMZN.VolumebyVenue()



if __name__ == '__main__':
    unittest.main()
  



import setuptools



required = [
    "pandas",
    "requests"
    ]

setuptools.setup(name = 'pyinvestor',
                 version = "0.0.2",
                 description = "python wrapper for the IEX API",
                 long_description = open('README.md').read().strip(),
                 author = 'SamurAI Sarl (Fabio Capela)',
                 author_email = 'fabio.capela@samurai.team',
                 url = 'https://github.com/SamurAi-sarl/PyInvestor',
                 packages = ['PyInvestor'],
                 install_requires = required,
                 keywords = ['finance', 'trading', 'stock', 'IEX', 'API', 'market'],
                 license = 'Apache-2.0',
                 zip_safe = False)

                 
